#!/usr/bin/perl -w

use lib "..";
use HotSaNIC;
use RRDs;

# get cmdline arguments (i.e. options passed by the daemon)
# and other global parameters
#
%ARGS=arg2opt(@ARGV);

dupe_control("start",$ARGS{"MODNAME"},"");

# read module-specific settings
#
undef @NETWORKS;
open (FILE,"settings") or dupe_control("die",$ARGS{"MODNAME"},"error - $!");
while (<FILE>) {
  ($var,$value)=parse_line($_);
  next if $var eq "";
  if ($var eq "LOGPATH") { $DNETLOG="$value/"; }
  if ($var eq "LOGNAME_RC5") { $LOG_RC5="$value"; }
  if ($var eq "LOGNAME_DES") { $LOG_DES="$value"; }
  if ($var eq "LOGNAME_CSC") { $LOG_CSC="$value"; }
  if ($var eq "LOGNAME_OGR") { $LOG_OGR="$value"; }
  if ($var eq "NETWORK") {
    ($name,$range)=split(/:/,$value);
    if ($range eq "*") { $OTHER=$name; }
    else { push @NETWORKS,$value; }
    }
  }
close FILE;

undef @DNETLOGFILES;
$rebuilding=0;
$databases=0;
$hasargs=0;

# list all databases that exist and assume all entries for the day to check
# to be zero as initialization. This is done to make sure ALL databases will
# be updated.
opendir(DIR,"rrd") || dupe_control("die",$ARGS{"MODNAME"},"error opening database dir - $!");
@DATABASES=grep(/.rrd/,readdir(DIR));
closedir(DIR);
undef %RESULTS;
foreach $file (@DATABASES) {
  $databases++;
  $testname=$file;
  if (index($file,"dummy")<0) {
    ($name) = split (/.rrd/,$file);
    foreach $type ("rc5","ogr","csc","des") { $RESULTS{$name,$type}=0; }
    }
  else { $rebuilding=1; }
  }

##### if no databases are found, rebuild the whole thing from scratch...
if ( $databases == 0 ) { rebuilddb($DNETLOG,$ARGS{"MODNAME"},$OTHER,$LOG_RC5,$LOG_DES,$LOG_CSC,$LOG_OGR,@NETWORKS); dupe_control("stop",$ARGS{"MODNAME"},""); }

  
##### find out when (days since 01-01-1970) databases were last modified
# last found database is being used to test...
(undef,undef,undef,undef,undef,undef,undef,undef,undef,$mtime,undef,undef,undef)=stat("rrd/".$testname);
$mtime=int($mtime/86400);

# only update database if last modified yesterday
# or if an argument has been given to us
# and a rebuild-process is not currently happening
#
if ( ($hasargs == 1) || ( ($rebuilding == 0) && ($mtime < int(time/86400)) && ((time%86400) > 10800) )) {

  @DNETLOGFILES=getlogfiles($DNETLOG,$ARGS{"MODNAME"},$LOG_RC5,$LOG_DES,$LOG_CSC,$LOG_OGR);
  ($logdate,undef,$entrydate1,$entrydate2,$sampletime)=getdatestrings(1);
  print $logdate,"     ",$entrydate1,"     ",$entrydate2,"     ",$sampletime,"\n";


print time," ",$ARGS{"MODNAME"},": ",$entrydate2,"\n";

# build array of files which contain that day's logs
#
  @FILES=grep(/$logdate/,@DNETLOGFILES);
  foreach $file (@FILES) {
    if(index($file,"rc5")>0) { $type="rc5"; }
    if(index($file,"ogr")>0) { $type="ogr"; }
    if(index($file,"csc")>0) { $type="csc"; }
    if(index($file,"des")>0) { $type="des"; }
    open(FILE,"$DNETLOG$file");
    $datefound=0;
    while(<FILE>) {
      chomp;
      ($date,$origip,undef,undef,$size)=split(",");
  
##### check if line has the corrosponding date and evaluate the line.
      if((index($date,$entrydate1)>=0) || (index($date,$entrydate2)>=0)) {
        $datefound=1;

	$name=ip2name($origip,$OTHER,@NETWORKS);
        $RESULTS{$name,$type}+=$size;
        }
# end of logfile-section with corrosponding date-entries
      else { if ($datefound == 1) {$datefound=2; last; }}
      }
    close(FILE);
    }

  print time," ",$ARGS{"MODNAME"},": distributed.net - 1 day ago:\n";
  print time," ",$ARGS{"MODNAME"},":                     rc5    des    csc                ogr\n";
  
  foreach $key (sort keys %RESULTS) {
    $name=substr($key,0,length($key)-4);
    if (index($key,"rc5") >=0) {
      $rc5=$RESULTS{$name,"rc5"}+0;
      $ogr=$RESULTS{$name,"ogr"}+0;
      $csc=$RESULTS{$name,"csc"}+0;
      $des=$RESULTS{$name,"des"}+0;
      printf "%u %s: %-17s %6u %6u %6u %18.0f\n",time,$ARGS{"MODNAME"},$name,$rc5,$des,$csc,$ogr;
      updatedb($name,$sampletime,$rc5,$ogr,$csc,$des);
      }
    }
  }
elsif ( $rebuilding > 0) { print time," ",$ARGS{"MODNAME"},": rebuilding - no update possible.\n"; }

dupe_control("stop",$ARGS{"MODNAME"},"");

sub rebuilddb {
  $|=1;
  my ($DNETLOG,$MODNAME,$OTHER,$LOG_RC5,$LOG_DES,$LOG_CSC,$LOG_OGR,@NETWORKS)=@_;
  my ($LOGNAME,%RESULTS,$DAEMONDIR,$lines,$start,@DNETLOGFILES,$log,$type,$datestr,$origip,$email,$size,$date,$name,$mon,$day,$year,$fulldate,$num,$daysago,$update,$sampletime,$rc5,$ogr,$csc,$des,$value);

  print time," ",$MODNAME,": databases missing - rebuilding...\n";
  open (FILE,">rrd/dummy.rrd");
  close FILE;

#  first of all parse configurationfile...
  open(CONF,"../settings") or dupe_control("die",$MODNAME,"error opening global configuration-file - $!\nPlease make sure there is a file named \"settings\" in your rrdgraph directory.");
  while (<CONF>) {
    chomp;
    if (index($_,"DAEMONDIR=") >= 0) { (undef,$DAEMONDIR) = split (/=/);}
    if (index($_,"LOGFILES=") >= 0) { (undef,$LOGNAME) = split (/=/);}
    }
  close CONF;

  ($DAEMONDIR,$LOGNAME)=strip_unwanted($DAEMONDIR,$LOGNAME);
  $LOGNAME=~ s/\$DAEMONDIR/$DAEMONDIR/g;
  if ($LOGNAME eq "") { $LOGNAME="."; }
  open (LOGFILE,">$LOGNAME/dnet-builddb");

  $lines=0;
  $start=time;
  @DNETLOGFILES=getlogfiles($DNETLOG,$MODNAME,$LOG_RC5,$LOG_DES,$LOG_CSC,$LOG_OGR);
  foreach $log (@DNETLOGFILES) {
    print LOGFILE "scanning ",$DNETLOG,$log," ...\n";
    if (index($log,"rc5") >=0) { $type="rc5"; }
    if (index($log,"ogr") >=0) { $type="ogr"; }
    if (index($log,"csc") >=0) { $type="csc"; }
    if (index($log,"des") >=0) { $type="des"; }
    open (FILE,"$DNETLOG$log");
    while (<FILE>) {
      chomp;
      $lines++;
      ($datestr,$origip,$email,undef,$size)=split(",");
      $name=ip2name($origip,$OTHER,@NETWORKS);
      ($date)=split(/ /,$datestr);
      if(index($date,"/") >=0) {
	($mon,$day,$year)=split(/\//,$date);
	$fulldate=("1$year"+1900).$mon.$day;
        }
      $RESULTS{"$fulldate $name $type"}+=$size;
      }
    close FILE;
    }
  $num=%RESULTS;
  print LOGFILE "\n",$lines," lines scanned in ",time-$start,"s\n",$num," entries in hash\n\n";
  $start=time;
  $lines=0;
  $update=0;
  for ($daysago=500;$daysago>0;$daysago--) {
    (undef,$date,undef,undef,$sampletime)=getdatestrings($daysago);
    foreach $value (@NETWORKS) {
      ($name)=split(/:/,$value);
      foreach $type ("rc5","ogr","csc","des") {
        $result{$type}=$RESULTS{"$date $name $type"};
        if (!defined $result{$type}) { $result{$type}=0; } else { $update=1; }
        }
      if ($update > 0) {
        printf LOGFILE "%-17s %12u %6u %6u %6u %18.0f\n",$name,$sampletime,$result{"rc5"},$result{"des"},$result{"csc"},$result{"ogr"};
        updatedb($name,$sampletime,$result{"rc5"},$result{"ogr"},$result{"csc"},$result{"des"});
        $lines++;
        }
      }
    }
  print LOGFILE "\n",$lines," database-updates in ",time-$start,"s\n";
  close LOGFILE;
  unlink "rrd/dummy.rrd";
  }

##### list all logfiles in logdir and one subdir deep
#
# @DNETLOGFILES will contain all plain ascii keyblock logfiles
# console-logs and packed logs should not be listed.
#
sub getlogfiles {
  my ($DNETLOG,$MODNAME,$LOG_RC5,$LOG_DES,$LOG_CSC,$LOG_OGR)=@_;
  my @SUBDIRS;

  opendir(DIR,$DNETLOG) || dupe_control("die",$MODNAME,"can't open dir $DNETLOG");
  my @dir=readdir(DIR);
  closedir(DIR);

  my @DNETLOGFILES=grep /$LOG_RC5|$LOG_DES|$LOG_CSC|$LOG_OGR/,@dir;

  foreach my $file (@dir) {
    if (( $file ne "." ) && ( $file ne ".." )) {
      if ( -d "$DNETLOG$file" ) { push @SUBDIRS,"$file/"; }
      }
    }

  foreach my $entry (@SUBDIRS) {
    opendir(DIR,"$DNETLOG$entry") || dupe_control("die",$MODNAME,"can't open dir $DNETLOG$entry");
    @dir=readdir DIR;
    closedir(DIR);


    foreach my $file (@dir) {
      if ( $file =~ /$LOG_RC5|$LOG_DES|$LOG_CSC|$LOG_OGR/ ) {
        push @DNETLOGFILES,"$entry$file";
	}
      }
    }
  return @DNETLOGFILES;
  }

##### get date-strings of "$days days ago" 
sub getdatestrings {
  my $days=shift;
  my $sampletime=(int(time/86400)-$days)*86400;
  (undef,undef,undef,my $mday,my $mon,my $year,undef,undef,undef) = localtime($sampletime);

  $year+=1900;
  $mon++;
  if ($mon < 10) { $mon="0$mon"; }
  if ($mday < 10) { $mday = "0".$mday; }

  my $yyyymm="$year$mon";
  my $yyyymmdd="$year$mon$mday";
  my $entrydate1="$mon/$mday/".substr($year,2,2);
  my $entrydate2="$year-$mon-$mday";
  return ($yyyymm,$yyyymmdd,$entrydate1,$entrydate2,$sampletime);
  }

##### convert IP number to a configured network or host.
sub ip2name {
  my ($ip,$OTHER,@NETWORKS)=@_;
  my $name="";
  foreach my $value (@NETWORKS) {
    my ($netname,$range)=split(/:/,$value);
    for my $subnet (split(/,/,$range)) {
      if (index("$ip.",$subnet) == 0) { $name=$netname; }
      }
    }
  if ( $name eq "" ) { $name=$OTHER; }
  return $name;
  }

##### update specific database
sub updatedb {
  my ($name,$sampletime,$rc5,$ogr,$csc,$des)=@_;
  my $ERROR;
  if ( ! -e "rrd/$name.rrd" ) { system("./makerrd","$name") }
  RRDs::update "rrd/$name.rrd",$sampletime.":".$rc5.":".$ogr.":".$csc.":".$des;  
  if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"}."unable to update `$name.rrd': $ERROR"); }
  }

