#!/usr/bin/perl -w

use lib "..";
use HotSaNIC;
use RRDs;

# get cmdline arguments (i.e. options passed by the daemon)
# and other global parameters
#
%ARGS=arg2opt(@ARGV);

dupe_control("start",$ARGS{"MODNAME"},"");

$OS_TYPE=$ARGS{"OSTYPE"};

# read module-specific settings
#

# build new database(s) if needed
  if ( ! -e "rrd/loadavg.rrd" ) { system("./makerrd","loadavg") }
  if ( ! -e "rrd/users.rrd" ) { system("./makerrd","users") }
  if ( ! -e "rrd/proc.rrd" ) { system("./makerrd","proc") }
  if ( ! -e "rrd/mem.rrd" ) { system("./makerrd","mem") }

# build CPU statistics
# in /proc/stat there is a CPU entry which provides us with the
# number of ticks the CPU was in all states ( user sys nice idle )
# order in /proc/stat:  usr nice system idle
# up to now. We simply have to gather this information and 
# calculate the actual percentage :)
# To do this we have to remember the last values of course!
#
# FreeBSD values can be obtained from "sysctl kern.cp_time".
# However, FreeBSD has 5 values ( user, nice, sys, int, idle )


$multicpu=0;

# read actual state
if ($OS_TYPE =~ /Linux/) {
  open(IN,"/proc/stat");
  while(<IN>) {
    chomp;
    if( index($_,"cpu") >=0 ) {
      ($cpu, $cpu1, $cpu2, $cpu3, $cpu4)=split;
      $cpu1{$cpu}=$cpu1;
      $cpu2{$cpu}=$cpu2;
      $cpu3{$cpu}=$cpu3;
      $cpu4{$cpu}=$cpu4;
      $cpu5{$cpu}=0;
      $cpu1old{$cpu}=0;
      $cpu2old{$cpu}=0;
      $cpu3old{$cpu}=0;
      $cpu4old{$cpu}=0;
      $cpu5old{$cpu}=0;
      if ($cpu eq "cpu1") { $multicpu=1; }
      }
    }
  close(IN);
  }
elsif ($OS_TYPE =~ /FreeBSD/) {
  # assumed order is: user nice sys intr idle
  $cpuraw = `sysctl kern.cp_time`;
  chomp $cpuraw;
  ($cpu, $cpu1, $cpu2, $cpu3, $cpu5, $cpu4)=split / /, $cpuraw;
  if ($cpu eq "kern.cp_time:") { $cpu = "cpu"; }
  $cpu1{$cpu}=$cpu1;
  $cpu2{$cpu}=$cpu2;
  $cpu3{$cpu}=$cpu3;
  $cpu4{$cpu}=$cpu5;
  $cpu5{$cpu}=$cpu4;
  $cpu1old{$cpu}=0;
  $cpu2old{$cpu}=0;
  $cpu3old{$cpu}=0;
  $cpu4old{$cpu}=0;
  $cpu5old{$cpu}=0;
  if ($cpu eq "cpu1") { $multicpu=1; }
  }
elsif ($OS_TYPE =~ /NetBSD/) {
  $cpuraw = `/sbin/sysctl kern.cp_time`;
  $cpuraw =~ /^.* user = (\d+), nice = (\d+), sys = (\d+), intr = (\d+), idle = (\d+)$/;
  $cpu="cpu";
  $cpu1{$cpu}=$1;
  $cpu2{$cpu}=$2;
  $cpu3{$cpu}=$3;
  $cpu4{$cpu}=$5;
  $cpu5{$cpu}=$4;
  $cpu1old{$cpu}=0;
  $cpu2old{$cpu}=0;
  $cpu3old{$cpu}=0;
  $cpu4old{$cpu}=0;
  $cpu5old{$cpu}=0;
  }
elsif ($OS_TYPE =~ /OpenBSD/) {
  # assumed order is: user nice sys intr idle
  $cpuraw = `sysctl kern.cp_time`;
  chomp $cpuraw;
  ($cpu, $cpu1, $cpu2, $cpu3, $cpu5, $cpu4)=split / /, $cpuraw;
  $cpu1{$cpu}=$cpu1;
  $cpu2{$cpu}=$cpu2;
  $cpu3{$cpu}=$cpu3;
  $cpu4{$cpu}=$cpu5;
  $cpu5{$cpu}=$cpu4;
  $cpu1old{$cpu}=0;
  $cpu2old{$cpu}=0;
  $cpu3old{$cpu}=0;
  $cpu4old{$cpu}=0;
  $cpu5old{$cpu}=0;
  if ($cpu eq "cpu1") { $multicpu=1; }
  }

# read last state and store actual state
# usr nice sys idle int
if ( -e "cpu.dat" ) {
  open(CPU,"cpu.dat");
  while (<CPU>) {
    $line=$_." 0 0 0 0 0 0";
    ($cpu, $cpu1old, $cpu2old, $cpu3old, $cpu4old, $cpu5old)=split / /,$line;
  
    if ($cpu !~ /cpu/) {
      $cpu5old=$cpu4old;
      $cpu4old=$cpu3old;
      $cpu3old=$cpu2old;
      $cpu2old=$cpu1old;
      $cpu1old=$cpu;
      $cpu="cpu";
      }

    $cpu1old{$cpu}=$cpu1old;
    $cpu2old{$cpu}=$cpu2old;
    $cpu3old{$cpu}=$cpu3old;
    $cpu4old{$cpu}=$cpu4old;
    $cpu5old{$cpu}=$cpu5old;
    }
  close(CPU);
  }

open(CPU,">cpu.dat");

foreach $cpu (keys %cpu1) {
  print CPU $cpu." ".$cpu1{$cpu}." ".$cpu2{$cpu}." ".$cpu3{$cpu}." ".$cpu4{$cpu}." ".$cpu5{$cpu}."\n";

# calculate everything....


  $all{$cpu}=($cpu1{$cpu}+$cpu2{$cpu}+$cpu3{$cpu}+$cpu4{$cpu}+$cpu5{$cpu})-($cpu1old{$cpu}+$cpu2old{$cpu}+$cpu3old{$cpu}+$cpu4old{$cpu}+$cpu5old{$cpu});

  $cpuusr{$cpu}=($cpu1{$cpu}-$cpu1old{$cpu})/$all{$cpu};
  $cpunic{$cpu}=($cpu2{$cpu}-$cpu2old{$cpu})/$all{$cpu};
  $cpusys{$cpu}=($cpu3{$cpu}-$cpu3old{$cpu})/$all{$cpu};
  $cpuidl{$cpu}=($cpu4{$cpu}-$cpu4old{$cpu})/$all{$cpu};
  $cpuint{$cpu}=($cpu5{$cpu}-$cpu5old{$cpu})/$all{$cpu};

  if ( ! -e "rrd/".$cpu.".rrd" ) { system("./makerrd",$cpu); }

  if ($OS_TYPE =~ /Linux/) {
    RRDs::update "rrd/".$cpu.".rrd", time.":".$cpuusr{$cpu}.":".$cpunic{$cpu}.":".$cpusys{$cpu}.":".$cpuidl{$cpu};
    if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update cpu.rrd: $ERROR"); }
    }
  elsif ($OS_TYPE =~ /FreeBSD/) {
    RRDs::update "rrd/".$cpu.".rrd", time.":".$cpuusr{$cpu}.":".$cpunic{$cpu}.":".$cpusys{$cpu}.":".$cpuidl{$cpu}.":".$cpuint{$cpu};
    if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update cpu.rrd: $ERROR"); }
    }
  elsif ($OS_TYPE =~ /NetBSD/) {
    RRDs::update "rrd/".$cpu.".rrd", time.":".$cpuusr{$cpu}.":".$cpunic{$cpu}.":".$cpusys{$cpu}.":".$cpuidl{$cpu}.":".$cpuint{$cpu};
    if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update cpu.rrd: $ERROR"); }
    }
  elsif ($OS_TYPE =~ /OpenBSD/) {
    RRDs::update "rrd/".$cpu.".rrd", time.":".$cpuusr{$cpu}.":".$cpunic{$cpu}.":".$cpusys{$cpu}.":".$cpuidl{$cpu}.":".$cpuint{$cpu};
    if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update cpu.rrd: $ERROR"); }
    }

# on single-cpu machines only the global database has to be updated.
  last if ($multicpu==0);

  }
close(CPU);

#####################################################################################
# here we go with the load-average value ...
#####################################################################################

if ($OS_TYPE =~ /Linux/) {
  open(IN,"/proc/loadavg"); ($load1,$load5,$load15)=split(/ /,<IN>); close(IN);
  }
elsif ($OS_TYPE =~ /FreeBSD/) {
  (undef,undef,$load1,$load5,$load15,undef)=split(/ /,`/sbin/sysctl vm.loadavg`);
  }
elsif ($OS_TYPE =~ /NetBSD/) {
  (undef,$load1,$load5,$load15)=split(/ /,`/sbin/sysctl vm.loadavg`);
  }
elsif ($OS_TYPE =~ /OpenBSD/) {
  (undef,undef,$load1,$load5,$load15)=split(/ /,`/usr/sbin/sysctl vm.loadavg`);
  }

RRDs::update "rrd/loadavg.rrd", time.":".$load15.":".$load5.":".$load1;
if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update loadavg.rrd: $ERROR"); }


#####################################################################################
# now the processes. We have to check how many processes are sleeping,
# running, zombieng spwapped to disc or stopped. To do this we simply
# have to walk through the /proc tree, check all "stat" files of each
# process-id subdir and increment the according state-counter.
#
($procslp,$procrun,$proczmb,$procstp,$procdsc)=(0,0,0,0,0);

if ($OS_TYPE =~ /Linux/) {
  opendir (DIR,"/proc");@dir=readdir (DIR);closedir(DIR);
  foreach $entry (@dir) {
    # check if entry is a numeric one
    if (($entry =~ /^\d+$/) && ( -e "/proc/$entry/stat" )) {
      open (STAT,"/proc/$entry/stat");$_=<STAT>;close(STAT);
      if (index($_," S ") >=0 ) { $procslp++ }
      elsif (index($_," D ") >=0 ) { $procdsc++ }
      elsif (index($_," R ") >=0 ) { $procrun++ }
      elsif (index($_," Z ") >=0 ) { $proczmb++ }
      elsif (index($_," T ") >=0 ) { $procstp++ }
      }
    }
  }
elsif ($OS_TYPE =~ /FreeBSD/) {
  # ok, this needs work doing to it. Unfortunately I'm not quite
  # familiar enough with FreeBSD to do this properly - any takers? - MJB

  $procslp=$procrun=$proczmb=$procstp=$procdsc=0;
  @states=`/bin/ps axostate`;
  foreach (@states) {
    $procslp++ if (/^S/);
    $procdsc++ if (/^D/);
    $procrun++ if (/^R/);
    $proczmb++ if (/^Z/);
    $procstp++ if (/^T/);
  }
}
elsif ($OS_TYPE =~ /NetBSD/) {
  $procslp=$procrun=$proczmb=$procstp=$procdsc=0;
  @states=`/bin/ps axostate`;
  foreach (@states) {
    $procslp++ if (/^S/);
    $procdsc++ if (/^D/);
    $procrun++ if (/^R/);
    $proczmb++ if (/^Z/);
    $procstp++ if (/^T/);
  }
}
elsif ($OS_TYPE =~ /OpenBSD/) {
  $procslp=$procrun=$proczmb=$procstp=$procdsc=0;
  @states=`/bin/ps axostate`;
  foreach (@states) {
    $procslp++ if (/^S/);
    $procdsc++ if (/^D/);
    $procrun++ if (/^R/);
    $proczmb++ if (/^Z/);
    $procstp++ if (/^T/);
  }
}

# print "$procslp,$procrun,$proczmb,$procstp,$procdsc\n";

RRDs::update "rrd/proc.rrd", time.":".$procslp.":".$procrun.":".$proczmb.":".$procstp.":".$procdsc;
if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update proc.rrd: $ERROR"); }


#####################################################################################
# users stats are not supported yet, i'm working on it ...
#
if ($OS_TYPE =~ /Linux/) {
  @users=`who`;
  ($tty,$pty,$pts)=(0,0,0);
  foreach (@users) {
    #  print $_;
    if (index($_," tty") >=0 ) { $tty++; }
    elsif (index($_," pty") >=0 ) { $pty++; }
    elsif (index($_," pts") >=0 ) { $pts++; }
    }
  }
elsif ($OS_TYPE =~ /FreeBSD/) {
  # XXX im not shure if this will work. PR
  @users=`who`;
  ($tty,$pty,$pts)=(0,0,0);
  foreach (@users) {
    #  print $_;
    if (index($_," ttyv") >=0 )    { $tty++; }
    elsif (index($_," ttyp") >=0 ) { $pty++; }
    elsif (index($_," pts") >=0 ) { $pts++; }
    }
  }
elsif ($OS_TYPE =~ /NetBSD/) {
  @users=`who`;
  ($tty,$pty,$pts)=(0,0,0);
  foreach (@users) {
    #  print $_;
    if (index($_," ttyE") >=0 )    { $tty++; }
    elsif (index($_," ttyp") >=0 ) { $pty++; }
    elsif (index($_," pts") >=0 )  { $pts++; }
    }
  }
elsif ($OS_TYPE =~ /OpenBSD/) {
  @users=`who`;
  ($tty,$pty,$pts)=(0,0,0);
  foreach (@users) {
    #  print $_;
    if (index($_," tty") >=0 ) { $tty++; }
    elsif (index($_," pty") >=0 ) { $pty++; }
    elsif (index($_," pts") >=0 ) { $pts++; }
    }
  }
# print time.":",$tty,":",$pty,":",$pts,"\n";

RRDs::update "rrd/users.rrd", time.":".$tty.":".$pty.":".$pts;
if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update users.rrd: $ERROR"); }



####################################################################################
# here we go with the memory and swapfile statistics...
#
if ($OS_TYPE =~ /Linux/) {
  open(IN,"/proc/meminfo");
  while(<IN>) {
    chomp;
    if (index($_,"Mem:")>=0) { (undef,$memtot,undef,$memfre,$memshr,$membuf,$memcac)=split; }
    elsif (index($_,"Swap:")>=0) { (undef,undef,$swpuse,$swpfre)=split; }
    }
  close(IN);

  $memshr=$memtot-$memfre-$membuf-$memcac;

  RRDs::update "rrd/mem.rrd", time.":".$memfre.":".$memshr.":".$membuf.":".$memcac.":".$swpfre.":".$swpuse;
  if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update `mem.rrd': $ERROR"); }

} elsif ($OS_TYPE =~ /FreeBSD/) {
  # FreeBSD sucks for this...
  (undef, $pagesize) = split(/ /, `sysctl vm.stats.vm.v_page_size`);
  (undef, $memtotl) = split(/ /, `sysctl vm.stats.vm.v_page_count`);    $memtotl *= $pagesize;
  (undef, $memfree) = split(/ /, `sysctl vm.stats.vm.v_free_count`);    $memfree *= $pagesize;
  (undef, $memwire) = split(/ /, `sysctl vm.stats.vm.v_wire_count`);    $memwire *= $pagesize;
  (undef, $memactv) = split(/ /, `sysctl vm.stats.vm.v_active_count`);  $memactv *= $pagesize;
  (undef, $meminac) = split(/ /, `sysctl vm.stats.vm.v_inactive_count`);$meminac *= $pagesize;
  (undef, $memcach) = split(/ /, `sysctl vm.stats.vm.v_cache_count`);   $memcach *= $pagesize;

  open(IN,"/usr/sbin/pstat -sk|");
  while(<IN>) {
    if (/Total/) { (undef, undef, $swpuse, $swpfre) = split; last; }
    }

  $swpuse*=1024;
  $swpfre*=1024;

  RRDs::update "rrd/mem.rrd", time.":".$memfree.":".$memwire.":".$memactv.":".$meminac.":".$memcach.":".$swpfre.":".$swpuse;
  if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update `mem.rrd': $ERROR"); }
} elsif ($OS_TYPE =~ /NetBSD/) {
  # hmm no sysctl for mem
  my $line = `/usr/bin/top -n|grep Memory`;
  chomp $line;
  $line =~ s/^Memory: (\d+\w) Act, (\d+\w) Inact, (\d+\w) Wired, (\d+\w) Free, (\d+\w) Swp, (\d+\w) Swp free$/$1 $2 $3 $4 $5 $6/;
  @results = split / /,$line;
  my ($memactv,$meminac,$memwire,$memfree,$swpuse,$swpfre) = HotSaNIC::convert_units(@results);
  #print STDERR "$memactv,$meminac,$memwire,$memfree,$swpuse,$swpfre\n";

  RRDs::update "rrd/mem.rrd", time.":".$memfree.":".$memwire.":".$memactv.":".$meminac.":".$swpfre.":".$swpuse;
  if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},"unable to update `mem.rrd': $ERROR"); }
} 

#
# users.rrd -> tty pty pts
# cpu.rrd -> cpuusr cpusys cpunic cpuidl
# loadavg.rrd -> load15 load5 load1
# proc.rrd -> procslp procrun proczmb procstp
# mem.rrd (Linux) -> memfre memshr membuf memcac swpfre swpuse
# mem.rrd (FreeBSD) -> memfree memwire memactv meminac memcach swpfre swpuse
# mem.rrd (NetBSD) -> memact meminac memwire memfree swpuse swpfree

dupe_control("stop",$ARGS{"MODNAME"},"");
