#!/usr/bin/perl

use lib "..";
use HotSaNIC;
use RRDs;
use Time::Local;
$|=1;

@months=("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
%MONTHS=(Jan=>0,Feb=>1,Mar=>2,Apr=>3,May=>4,Jun=>5,Jul=>6,Aug=>7,Sep=>8,Oct=>9,Nov=>10,Dec=>11);

# get cmdline arguments (i.e. options passed by the daemon)
# and other global parameters
#
%ARGS=arg2opt(@ARGV);

dupe_control("start",$ARGS{"MODNAME"},"");

# read module-specific settings
#
undef @check;
open (FILE,"settings") or dupe_control("die",$ARGS{"MODNAME"},": error - $!");
while (<FILE>) {
  ($var,$value)=parse_line($_);
  if ($var eq "LOGPATH") {
    @dummy=split /\//,$value;
    $logname=pop @dummy;
    $logdir=join "/",@dummy;
    next;
    }
  if ($var eq "INTERVAL") { $INTERVAL=$value; next; }
  if ($var eq "ITEM") {
    ($item,$name)=split(/:/,$value);
    push @check,$item;
    }
  }

# $logdir="/var/log/httpd";
# $logname="access_log";
# @check=("default.ida","root.exe","cmd.exe");

#print join ("  ",@check),"\n";
#print $logdir,"\n";
#print $logname,"\n";

if (! defined $INTERVAL) { $INTERVAL=600; }
$now=time;
(undef,undef,undef,undef,undef,undef,undef,undef,undef,$mtime,undef,undef,undef)=stat("lastcheck.dat");
if ($mtime+$INTERVAL > $now) { dupe_control("stop",$ARGS{"MODNAME"},""); }
utime $now,$now,"lastcheck.dat";

# read log-directory and list all http-logs
#
opendir DIR,$logdir;
@dir=grep /^$logname/,readdir DIR;
closedir DIR;

# check lastcheck.dat to know which file was parsed last
#
if (-e "lastcheck.dat") {
  open LAST,"lastcheck.dat";
  $firstline=<LAST>;
  $linenum=<LAST>;
  $seconds1=<LAST>;
  chomp $seconds1;
  chomp $linenum;
  close LAST;
  }
else { $firstline=""; $linenum=0; }


# search last parsed logfile and logs that are newer than last parsed one
#
undef @files;
$found=0;
foreach $file (sort @dir) {
  open FILE,"$logdir/$file";
  $first=<FILE>;
  close FILE;
  unshift @files,$file;
  if ($firstline eq $first) { last; }
  }

# parse logs
#
#$seconds1=0;
$isfirstfile=1;
if ($ARGS{"DEBUGLEVEL"} >=0) { print "parsing files\n"; }
foreach $file (@files) {
  if ($ARGS{"DEBUGLEVEL"} >=0) { print "parsing ",$file,"\n"; }
  $firstline="";
  open FILE,"$logdir/$file";
  $firstline=<FILE>;
  if ($isfirstfile==1) {
    if ($ARGS{"DEBUGLEVEL"} >=0) { print "seeking to pos. $linenum\n"; }
    seek FILE,$linenum,0;
    $isfirstfile=0;
    }
  while (<FILE>) {
    chomp;

    # evaluate time of log-entry
    (undef,$datestr)=split /[\[\]]/;
    ($date,$hour,$min,$sec)=split /:/,$datestr;
    (undef,$hrcorrection)=split / /,$datestr;
    ($day,$month,$year)=split /\//,$date;
    $mon=$MONTHS{$month};
    $hour*=1;
    $day*=1;
    $seconds=timelocal(0,0,$hour,$day,$mon,$year)-$hrcorrection*36;

    if ($ARGS{"DEBUGLEVEL"} >=1) { print "$year $mon $day $hour:$min  -  $seconds $hrcorrection  -  "; }

    # check if another hour started and save sampling results for last hour
    if ($seconds1 < $seconds) {
      if ($ARGS{"DEBUGLEVEL"} >=0) { print time," ",$ARGS{"MODNAME"},": save for $seconds1:"; }
      foreach $item (@check) {
        updatedb($item,$seconds1,$COUNT{$item});
        }
      if ($ARGS{"DEBUGLEVEL"} >=0) { print "\n"; }
      if (($seconds1+3600 < $seconds) && ($seconds1 > 0)) {
        for ($secs=$seconds1+3600;$secs<$seconds;$secs+=3600) {
          (undef,undef,$fhour,$fday,$fmon,$fyear,undef,undef,undef) = localtime($secs);
          $fyear+=1900;
          if ($ARGS{"DEBUGLEVEL"} >=0) { print time," ",$ARGS{"MODNAME"},": save for $secs:"; }
          foreach $item (@check) {
            updatedb($item,$secs,0);
            }
          if ($ARGS{"DEBUGLEVEL"} >=0) { print "\n"; }
          }
        }
      open LAST,">lastcheck.dat";
      print LAST "$firstline",tell(FILE),"\n",$seconds1,"\n";
      close LAST;
      $seconds1=$seconds;
      foreach $item (@check) { $COUNT{$item}=0; }
      }

    # check if any item is there
    foreach $item (@check) { if (index($_,"/$item?") >= 0) { $COUNT{$item}++; } }
    }
  close FILE;
  }

dupe_control("stop",$ARGS{"MODNAME"},"");

##### update specific database
sub updatedb {
  my ($name,$sampletime,$value)=@_;
  my $ERROR;
  if ($ARGS{"DEBUGLEVEL"} >=0) { print " ",$name,":$value"; }
  if ( ! -e "rrd/$name.rrd" ) { system("./makerrd","$name") }
  if ((defined $name) && (defined $sampletime) && (defined $value)) {
    RRDs::update "rrd/$name.rrd",$sampletime.":".$value;
    if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},": unable to update `$name.rrd': $ERROR"); }
    }
  }

