#!/usr/bin/perl -w

use diagnostics;
use FindBin;
use lib ".";
use lib $FindBin::RealBin;
use HotSaNIC;

$|=1;

$NOW=`date +%Y%m%d-%H%M%S`;

$THISDIR=`pwd`;
chomp $THISDIR;

$args=lc join "",@ARGV;
if ( index($args,"y") >=0 ) { $ALLYES="y"; }
else { $ALLYES="" };

$RUN="";
$SHOW="";

write_rrdgraph_script();
if ( ! -e "settings.all" ) { mkdir "settings.all",0755; }
if ( ! -e "backup" ) { mkdir "backup",0755; }
configure_modules();
configure_main();
if ( -e "settings.new" ) {
  system "mv -f settings backup/settings-$NOW";
  system "mv -f settings.new settings";
  }
system "cd settings.all; ln -sf ../settings main";
print "\nNow adapt all settings files to satisfy your needs.\nThey are all linked to the directory  ./settings.all .\n\n";
exit 0;



sub write_rrdgraph_script {
  open OUT,">rrdgraph";
  print OUT "#!/bin/sh
# chkconfig: 12345 55 45
# description:  System statistics

cd \"$THISDIR\"

. ./settings

case \$1 in
  start)
    echo \"Starting rrdtimer...\"
    ./rrdtimer Dp
    ;;
  stop)
    echo \"Stopping rrdtimer...\"
    PID=`cat \$PIDFILE`
    for nn in \$PID ; do kill \$nn; done
    ./clearall CLEAR_COUNTERS
    ;;
  restart)
    ./rrdgraph stop
    ./rrdgraph start
    ;;
  esac
";
  close OUT;
  chmod 0755,"rrdgraph";
  }


sub configure_modules {

  opendir DIR,".";
  @mods=grep /data-/,readdir DIR;
  closedir DIR;

  foreach $dir (@mods) {
    ($modname=uc $dir) =~ s/DATA-//g;
    $input=$ALLYES;
    if ($input eq "") {
      print "\n\nModule found: $modname\n----------------------------------------\nDo you want to use this module ? (Y)es / (n)o > ";
      while ($input eq "") {
        $input=lc <STDIN>;
        chomp $input;
        if ($input eq "") { $input="y"; }
        if (($input ne "y") && ($input ne "n")) {
          print "Please answer \"y\" or \"n\" > ";
          $input="";
          }
        }
      }
  
    if ($input eq "y") {
    if ( defined $RUN ) { $RUN=$RUN." ".lc $modname; } else { $RUN=lc $modname; }
      $input=$ALLYES;
      if ($input eq "") {
        print "Do you want to show this module's graphs on the webpage ? (Y)es / (n)o > ";
        while ($input eq "") {
          $input=lc <STDIN>;
          chomp $input;
          if ($input eq "") { $input="y"; }
          if (($input ne "y") && ($input ne "n")) {
            print "Please answer \"y\" or \"n\" > ";
            $input="";
            }
          }
	}
      if ($input eq "y") {
	if (defined $SHOW) { $SHOW=$SHOW." ".lc $modname; } else { $SHOW=lc $modname; }
        }
      print "\nsetting up $dir ...\n";
      system "cd $dir; ./setup; cd ../settings.all; ln -sf ../$dir/settings $dir";
      print "\n----------------------------------------\n";
      }
    }
  }



sub configure_main {

  print "\n\n\n";

  $file=".settings.default";
  $newfile="settings";
  if ( -e "settings" ) {
    $newfile="settings.new";
    @CONF=read_settings(".");
    foreach (@CONF) {
      ($var,$val)=parse_line($_);
      $VARS{$var}=$val;
      }
    }
  open FILE,$file;
  open NEWFILE,">$newfile";

  while (<FILE>) {
    chomp;
    
    ($var,$value,$comment)=parse_line($_);

    if (defined $VARS{$var}) { $value=$VARS{$var}; }
    if ($var eq "RUN") { ($value=$RUN) =~ s/^ *//; }
    if ($var eq "SHOW") { ($value=$SHOW) =~ s/^ *//; }

    if ( ($value eq "" ) || (lc $value eq "not configured") ) {

      if ($var eq "OSTYPE") {
        print "  checking for OS-type (current: ",$value,")\n";
        $value=identify_os_type();
        }

      if ($var eq "DAEMONDIR") {
        print "  checking path to \"rrdtimer\" (current: ",$value,")\n";
        if ( -e "$THISDIR/rrdtimer" ) { $value=$THISDIR; }
        }

      if ($var eq "BINPATH") {
        print "  checking path to \"rrdtool\" (current: ",$value,")\n";
        @found=`locate bin\/rrdtool`;
        if (@found) {
          foreach $nn (@found) { $nn=~ s/\/rrdtool$//g; }
          $value=choose($value,"rrdtool could not be found. If you already installed it, please edit the settings file.",@found);
          }
        }

      if ($var eq "CONVERTPATH") {
        print "  checking path to \"convert\" from ImageMagick (current: ",$value,")\n";
        @found=`locate bin\/convert|grep convert\$`;
        if (@found) {
          $value=choose($value,"convert could not be found. If ImageMagick package is already installed, please edit the settings file.",@found);
          }
        }

      if ($var eq "SNMPWALK") {
        print "  checking path to \"snmpwalk\" (current: ",$value,")\n";
        @found=`locate bin\/snmpwalk`;
        if (@found) {
          $value=choose($value,"snmpwalk could not be found. If you already installed it, please edit the settings file.",@found);
          }
        }

      if ($var eq "SNMPGET") {
        print "  checking path to \"snmpget\" (current: ",$value,")\n";
        @found=`locate bin\/snmpget|grep "snmpget\$"`;
        if (@found) {
          $value=choose($value,"snmpget could not be found. If you already installed it, please edit the settings file.",@found);
          }
        }

      }

    if ($var ne "") { print NEWFILE "$var=\"$value\""; print "$var=\"$value\"\n"; }
    if ($comment ne "") { $comment="" if $comment eq " "; print NEWFILE "#$comment"; }
    print NEWFILE "\n";
      
#     if ( ($value ne " ") && ($var !~ /^#/) ) { print "$var=\"$value\"\n"; }
#     } else { $var=""; }
#   if ($value ne " ") { print NEWFILE $var,"=\"",$value,"\"\n"; }
#   else { print NEWFILE $var,"\n"; }
    }
  close FILE;
  close NEWFILE;
  }



sub choose {
  my ($value,$errormsg,@ITEMS)=@_;

  $ok=0;
  if (@ITEMS) {
    foreach $nn (@ITEMS) {
      chomp $nn;
      if ( $nn eq $value ) { $ok=1; }
      }
    if ($ok == 0) {
    if ($#ITEMS == 0) {
      print "detected: ",$ITEMS[0],"\n";
      print "is this corrrect? (y)";
      $input=<STDIN>;
      chomp $input;
      $input=lc $input;
      if ( ($input eq "") || ($input eq "y") ) { $ok=1; $value=$ITEMS[0]; }
      } else {
        print "the following possibilities have been detected:\n";
        $pos=0;
        foreach $nn (@ITEMS) {
          chomp $nn;
          print $pos++,"  ",$nn,"\n";
          }
        $input=-1;
        while ( ! ( ($input >= 0) && ($input < $pos)) ) {
          print "\nwhich one shall i use (0=default ... ",$pos-1,") ? >";
          $input=<STDIN>;
          chomp$input;
          if ($input!~ /^[0-9]+$/) { $input=0; }
          }
        $value=$ITEMS[$input];
        }
      }
    }
  else { print "\nERROR: ",$errormsg,"\n\n"; }
  return $value;
  }

