#!/usr/bin/perl

if ((@ARGV != 3) && (@ARGV != 7)) {
  print "
USAGE:
warnings.pl <module> <database> <data-row-number> [MAX|MIN|AVERAGE] <timespan> [lt|eq|gt] <threshold>

  <module>            name of the module to query (you have to be in the HotSaNIC main directory)
  
  <database>          name of the database to query

  <data-row-number>   number of the database-row
                      \"?\" to show row names

  [MAX|MIN|AVERAGE]   consolidation function of the data-row you like to query

  <timespan>          timespan backwards from now in seconds

  [lt|eq|gt]          threshold functions
                        lt -> test if values are less than <threshold>
			eq -> test if values are equal <threshold>
			gt -> test if values are greater than <threshold>

  <threshold>         threshold value

  example:

   warnings.pl traffic eth0 1 MAX 120 gt 100000
     generates a warning message if the incoming traffic (row 1) of the traffic on eth0 exceeded 100000 bytes/s over the last 2 minutes

   warnings.pl system cpu 4 MAX 120 lt 0.3
     generates a warning message if the idle-time (row 4) of your CPU was constantly lower than 30% over the last 2 minutes
\n";
exit;
  }

($module,$db,$row,$consolidation,$time,$func,$threshold)=@ARGV;

if (@ARGV == 3) {
  @lines=`rrdtool fetch data-$module/rrd/$db.rrd -s -1 MAX`;
  print @lines;
  foreach (split $lines[0]) {
    print;
    print "\n";
  }
  exit;
}

@lines=`rrdtool fetch data-$module/rrd/$db.rrd -s -$time $consolidation`;

$caught=0;
$num=-1;
$vmin=999999999999999999;
$vmax=0;

if ($func eq "gt") { $function="higher than"; }
elsif ($func eq "eq") { $function="equal"; }
elsif ($func eq "lt") { $function="lower than"; }

foreach (@lines) {
  if ($num == -1) {
    chomp;
    @names=split;
    $num++;
    next;
    }
  if ((! /nan/) && ( /:/)) {
    chomp;
    $num++;
    @items=split;
    $val=@items[$row]*1;
    $vsum+=$val;
    $vmin=$val if $vmin>$val;
    $vmax=$val if $vmax<$val;
    if (($func eq "gt") && ($val>$threshold)) { $caught++; }
    elsif (($func eq "eq") && ($val==$threshold)) { $caught++; }
    elsif (($func eq "lt") && ($val<$threshold)) { $caught++; }
    }
  }

$vavg=$vsum/$num;

if ($num == $caught) {
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  $year+=1900;
  $dow=("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday")[$wday];
  if ($isdst) { $dst="(DST)"; } else { $dst=""; }
  $sec="0".$sec if $sec<10;
  $min="0".$min if $min<10;
  $hour="0".$hour if $hour<10;
  print "\n";
  print "THRESHOLD WARNING!\n";
  print "\n";
  print "Date    : $dow, $year-$mon-$mday\n";
  print "Time    : $hour:$min:$sec $dst\n";
  print "Module  : $module\n";
  print "data-row: ",@names[$row-1],"\n";
  print "\n";
  print $consolidation," value constantly ",$function," ",$threshold," over the last ",$time," seconds.\n";
  printf "minimum: %12.2f\n",$vmin;
  printf "average: %12.2f\n",$vavg;
  printf "maximum: %12.2f\n",$vmax;
  print "\n";
  }

